@@ -19,8 +19,11 @@
 package org.apache.sedona.common.utils;
 
 import com.sun.media.imageioimpl.common.BogusColorSpace;
+import org.apache.commons.lang3.tuple.Pair;
+import org.apache.sedona.common.Functions;
 import org.apache.sedona.common.FunctionsGeoTools;
 import org.apache.sedona.common.raster.RasterAccessors;
+import org.apache.sedona.common.raster.RasterEditors;
 import org.geotools.coverage.Category;
 import org.geotools.coverage.CoverageFactoryFinder;
 import org.geotools.coverage.GridSampleDimension;
@@ -57,9 +60,7 @@ import java.awt.image.DataBuffer;
 import java.awt.image.Raster;
 import java.awt.image.RenderedImage;
 import java.awt.image.WritableRaster;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.List;
+import java.util.*;
 
 /**
  * Utility functions for working with GridCoverage2D objects.
@@ -80,14 +81,124 @@ public class RasterUtils {
         return create(raster, gridGeometry, bands, null);
     }
 
+    /**
+     *
+     * @param raster WriteableRaster to be used while creating the new raster.
+     * @param gridGeometry2D gridGeometry2D to be used while cloning
+     * @param bands bands to be used while cloning
+     * @param referenceRaster referenceRaster to clone from
+     * @param noDataValue noDataValue (if any) to be applied to the bands. If provided null. bands are unchanged.
+     * @param keepMetadata if passed true, clone all possible metadata from the referenceRaster.
+                keepMetaData controls the presence/absence of the following metadata of the referenceRasterObject:
+                    raster name: Name of the raster (GridCoverage2D level)
+                    raster properties: A Map of raster and image properties combined.
+                    raster sources
+     * @return A cloned raster
+     */
+    public static GridCoverage2D clone(WritableRaster raster, GridGeometry2D gridGeometry2D, GridSampleDimension[] bands, GridCoverage2D referenceRaster, Double noDataValue, boolean keepMetadata) {
+        Map propertyMap = null;
+        if (keepMetadata) {
+            propertyMap = referenceRaster.getProperties();
+        }
+        ColorModel originalColorModel = referenceRaster.getRenderedImage().getColorModel();
+        if (Objects.isNull(gridGeometry2D)) {
+            gridGeometry2D = referenceRaster.getGridGeometry();
+        }
+        int numBand = raster.getNumBands();
+        int rasterDataType = raster.getDataBuffer().getDataType();
+        ColorModel colorModel;
+        if (originalColorModel.isCompatibleRaster(raster)) {
+            colorModel = originalColorModel;
+        }else {
+            final ColorSpace cs = new BogusColorSpace(numBand);
+            final int[] nBits = new int[numBand];
+            Arrays.fill(nBits, DataBuffer.getDataTypeSize(rasterDataType));
+            colorModel = new ComponentColorModel(cs, nBits, false, true, Transparency.OPAQUE, rasterDataType);
+        }
+        if (noDataValue != null) {
+            GridSampleDimension[] newBands = new GridSampleDimension[numBand];
+            for (int k = 0; k < numBand; k++) {
+                if (bands != null) {
+                    newBands[k] = createSampleDimensionWithNoDataValue(bands[k], noDataValue);
+                } else {
+                    newBands[k] = createSampleDimensionWithNoDataValue("band_" + k, noDataValue);
+                }
+            }
+            bands = newBands;
+        }
+
+        GridCoverage2D[] referenceRasterSources = keepMetadata ? referenceRaster.getSources().toArray(new GridCoverage2D[0]) : null;
+        CharSequence rasterName = keepMetadata ? referenceRaster.getName() : "genericCoverage";
+
+        final RenderedImage image = new BufferedImage(colorModel, raster, false, null);
+        return gridCoverageFactory.create(rasterName, image, gridGeometry2D, bands, referenceRasterSources, propertyMap);
+
+    }
+
+    public static GridCoverage2D clone(WritableRaster raster, GridSampleDimension[] bands, GridCoverage2D referenceRaster, Double noDataValue, boolean keepMetadata) {
+        return RasterUtils.clone(raster, null, bands, referenceRaster, noDataValue, keepMetadata);
+    }
+
+    public static GridCoverage2D clone(RenderedImage image, GridSampleDimension[] bands, GridCoverage2D referenceRaster, Double noDataValue, boolean keepMetadata) {
+        return RasterUtils.clone(image, null, bands, referenceRaster, noDataValue, keepMetadata);
+    }
+
+    /**
+     *
+     * @param image Rendered image to create the raster from
+     * @param gridGeometry2D gridGeometry2D to be used while cloning
+     * @param bands bands to be used while cloning
+     * @param referenceRaster referenceRaster to clone from
+     * @param noDataValue noDataValue (if any) to be applied to the bands. If provided null. bands are unchanged.
+     * @param keepMetadata if passed true, clone all possible metadata from the referenceRaster.
+        keepMetaData controls the presence/absence of the following metadata of the referenceRasterObject:
+            raster name: Name of the raster (GridCoverage2D level)
+            raster properties: A Map of raster and image properties combined.
+            raster sources
+     * @return A cloned raster
+     */
+    public static GridCoverage2D clone(RenderedImage image, GridGeometry2D gridGeometry2D, GridSampleDimension[] bands, GridCoverage2D referenceRaster, Double noDataValue, boolean keepMetadata) {
+        int numBand = image.getSampleModel().getNumBands();
+        if (Objects.isNull(gridGeometry2D)) {
+            gridGeometry2D = referenceRaster.getGridGeometry();
+        }
+        if (noDataValue != null) {
+            GridSampleDimension[] newBands = new GridSampleDimension[numBand];
+            for (int k = 0; k < numBand; k++) {
+                if (bands != null) {
+                    newBands[k] = createSampleDimensionWithNoDataValue(bands[k], noDataValue);
+                } else {
+                    newBands[k] = createSampleDimensionWithNoDataValue("band_" + k, noDataValue);
+                }
+            }
+            bands = newBands;
+        }
+        GridCoverage2D[] referenceRasterSources = keepMetadata ? referenceRaster.getSources().toArray(new GridCoverage2D[0]) : null;
+        Map propertyMap = null;
+        if (keepMetadata) {
+            propertyMap = referenceRaster.getProperties();
+        }
+        CharSequence rasterName = keepMetadata ? referenceRaster.getName() : "genericCoverage";
+        return gridCoverageFactory.create(rasterName, image, gridGeometry2D, bands, referenceRasterSources, propertyMap);
+    }
+
     /**
      * Create a new empty raster from the given WritableRaster object.
      * @param raster The raster object to be wrapped as an image.
      * @param gridGeometry The grid geometry of the raster.
      * @param bands The bands of the raster.
+     * @param noDataValue the noDataValue (if any) to be applied to all bands. If provided null, bands are unchanged.
+     *                     keepMetaData controls the presence/absence of the following metadata of the referenceRasterObject:
+     *             raster name: Name of the raster (GridCoverage2D level)
+     *             raster properties: A Map of raster and image properties combined.
+     *             raster sources
      * @return A new GridCoverage2D object.
      */
     public static GridCoverage2D create(WritableRaster raster, GridGeometry2D gridGeometry, GridSampleDimension[] bands, Double noDataValue) {
+      return create(raster, gridGeometry, bands, noDataValue, null);
+    }
+
+    public static GridCoverage2D create(WritableRaster raster, GridGeometry2D gridGeometry, GridSampleDimension[] bands, Double noDataValue, Map properties) {
         int numBand = raster.getNumBands();
         int rasterDataType = raster.getDataBuffer().getDataType();
 
@@ -113,7 +224,7 @@ public class RasterUtils {
         }
 
         final RenderedImage image = new BufferedImage(colorModel, raster, false, null);
-        return gridCoverageFactory.create("genericCoverage", image, gridGeometry, bands, null, null);
+        return gridCoverageFactory.create("genericCoverage", image, gridGeometry, bands, null, properties);
     }
 
     public static GridCoverage2D create(RenderedImage image, GridGeometry2D gridGeometry, GridSampleDimension[] bands, Double noDataValue) {
@@ -263,6 +374,27 @@ public class RasterUtils {
         return (AffineTransform2D) crsTransform;
     }
 
+    /**
+     * Translate an affine transformation by a given offset.
+     * @param affine the affine transformation
+     * @param offsetX the offset in x direction
+     * @param offsetY the offset in y direction
+     * @return the translated affine transformation
+     */
+    public static AffineTransform2D translateAffineTransform(AffineTransform2D affine, int offsetX, int offsetY) {
+        double ipX = affine.getTranslateX();
+        double ipY = affine.getTranslateY();
+        double scaleX = affine.getScaleX();
+        double scaleY = affine.getScaleY();
+        double skewX = affine.getShearX();
+        double skewY = affine.getShearY();
+
+        // Move the origin using the affine transformation, and leave scale and skew unchanged.
+        double newIpX = ipX + offsetX * scaleX + offsetY * skewX;
+        double newIpY = ipY + offsetX * skewY + offsetY * scaleY;
+        return new AffineTransform2D(scaleX, skewY, skewX, scaleY, newIpX, newIpY);
+    }
+
     public static Point2D getWorldCornerCoordinates(GridCoverage2D raster, int colX, int rowY) throws TransformException {
         return raster.getGridGeometry().getGridToCRS2D(PixelOrientation.UPPER_LEFT).transform(new GridCoordinates2D(colX - 1, rowY - 1), null);
     }
@@ -332,19 +464,64 @@ public class RasterUtils {
         return geometry;
     }
 
+    /**
+     * If the raster has a CRS, then it transforms the geom to the raster's CRS.
+     * If any of the inputs, raster or geom doesn't have a CRS, it defaults to 4326.
+     * @param raster
+     * @param geom
+     * @return
+     * @throws FactoryException
+     */
+    public static Pair<GridCoverage2D, Geometry> setDefaultCRSAndTransform(GridCoverage2D raster, Geometry geom) throws FactoryException {
+        int rasterSRID = RasterAccessors.srid(raster);
+        int geomSRID = Functions.getSRID(geom);
+
+        if (rasterSRID == 0) {
+            raster = RasterEditors.setSrid(raster, 4326);
+            rasterSRID = RasterAccessors.srid(raster);
+        }
+
+        if (geomSRID == 0) {
+            geom = Functions.setSRID(geom, 4326);
+            geomSRID = Functions.getSRID(geom);
+        }
+
+
+        if (rasterSRID != geomSRID) {
+            // implicitly converting roi geometry CRS to raster CRS
+            geom = convertCRSIfNeeded(geom, raster.getCoordinateReferenceSystem());
+            // have to set the SRID as RasterUtils.convertCRSIfNeeded doesn't set it even though the geometry is in raster's CRS
+            geom = Functions.setSRID(geom, RasterAccessors.srid(raster));
+        }
+        return Pair.of(raster, geom);
+    }
+
+    /**
+     * Converts data types to data type codes
+     * @param s pixel data type possible values {D, I, B, F, S, US} <br><br>
+     *          Update: add support to convert RS_BandPixelType data type string to data type code possible values: <br>
+     *          {REAL_64BITS, SIGNED_32BITS, UNSIGNED_8BITS, REAL_32BITS, SIGNED_16BITS, UNSIGNED_16BITS}
+     * @return Data type code
+     */
     public static int getDataTypeCode(String s) {
-        switch (s.toLowerCase()) {
-            case "d":
+        switch (s.toUpperCase()) {
+            case "D":
+            case "REAL_64BITS":
                 return 5;
-            case "i":
+            case "I":
+            case "SIGNED_32BITS":
                 return 3;
-            case "b":
+            case "B":
+            case "UNSIGNED_8BITS":
                 return 0;
-            case "f":
+            case "F":
+            case "REAL_32BITS":
                 return 4;
-            case "s":
+            case "S":
+            case "SIGNED_16BITS":
                 return 2;
-            case "us":
+            case "US":
+            case "UNSIGNED_16BITS":
                 return 1;
         }
         return 5; // defaulting to double
@@ -408,7 +585,7 @@ public class RasterUtils {
             sampleDimensions[numBand - 1] = new GridSampleDimension("band" + numBand);
         }
         // Construct a GridCoverage2D with the copied image.
-        return create(wr, gridCoverage2D.getGridGeometry(), sampleDimensions);
+        return clone(wr, gridCoverage2D.getGridGeometry(), sampleDimensions, gridCoverage2D, null, true);
     }
 
     public static GridCoverage2D copyRasterAndAppendBand(GridCoverage2D gridCoverage2D, Number[] bandValues) {
@@ -443,10 +620,26 @@ public class RasterUtils {
         } else if (noDataValue != null) {
             sampleDimensions[bandIndex - 1] = createSampleDimensionWithNoDataValue(sampleDimension, noDataValue);
         }
-        return create(wr, gridCoverage2D.getGridGeometry(), sampleDimensions);
+        return clone(wr, gridCoverage2D.getGridGeometry(), sampleDimensions, gridCoverage2D, null, true);
     }
 
     public static GridCoverage2D copyRasterAndReplaceBand(GridCoverage2D gridCoverage2D, int bandIndex, Number[] bandValues) {
         return copyRasterAndReplaceBand(gridCoverage2D, bandIndex, bandValues, null, false);
     }
+
+    /**
+     * Check if the two rasters are of the same shape
+     * @param raster1
+     * @param raster2
+     */
+    public static void isRasterSameShape(GridCoverage2D raster1, GridCoverage2D raster2) {
+        int width1 = RasterAccessors.getWidth(raster1), height1 = RasterAccessors.getHeight(raster1);
+        int width2 = RasterAccessors.getWidth(raster2), height2 = RasterAccessors.getHeight(raster2);
+
+        if (width1 != width2 && height1 != height2) {
+            throw new IllegalArgumentException(String.format("Provided rasters are not of same shape. \n" +
+                    "First raster having width of %d and height of %d. \n" +
+                    "Second raster having width of %d and height of %d", width1, height1, width2, height2));
+        }
+    }
 }
