@@ -40,6 +40,12 @@ from sedona.utils import KryoSerializer
 from sedona.utils import SedonaKryoRegistrator
 from sedona.register import SedonaRegistrator
 from sedona.spark.SedonaContext import SedonaContext
-from sedona.maps.SedonaKepler import SedonaKepler
-from sedona.maps.SedonaPyDeck import SedonaPyDeck
 from sedona.raster_utils.SedonaUtils import SedonaUtils
+try:
+    from sedona.maps.SedonaKepler import SedonaKepler
+except:
+    print('Skipping SedonaKepler import, verify if keplergl is installed')
+try:
+    from sedona.maps.SedonaPyDeck import SedonaPyDeck
+except:
+    print('Skipping SedonaPyDeck import, verify if pydeck is installed')
