@@ -153,7 +153,7 @@ case class ST_NDims(inputExpressions: Seq[Expression])
   * @param inputExpressions
   */
 case class ST_Buffer(inputExpressions: Seq[Expression])
-  extends InferredExpression(Functions.buffer _) {
+  extends InferredExpression(inferrableFunction2(Functions.buffer), inferrableFunction3(Functions.buffer)) {
 
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
@@ -219,7 +219,8 @@ case class ST_Centroid(inputExpressions: Seq[Expression])
   * @param inputExpressions
   */
 case class ST_Transform(inputExpressions: Seq[Expression])
-  extends InferredExpression(inferrableFunction4(FunctionsGeoTools.transform)) {
+  extends InferredExpression(inferrableFunction4(FunctionsGeoTools.transform), inferrableFunction3(FunctionsGeoTools.transform),
+                              inferrableFunction2(FunctionsGeoTools.transform)) {
 
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
@@ -259,7 +260,7 @@ case class ST_MakeValid(inputExpressions: Seq[Expression])
   * @param inputExpressions
   */
 case class ST_IsValid(inputExpressions: Seq[Expression])
-  extends InferredExpression(Functions.isValid _) {
+  extends InferredExpression(inferrableFunction2(Functions.isValid), inferrableFunction1(Functions.isValid)) {
 
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
@@ -509,6 +510,20 @@ case class ST_LineInterpolatePoint(inputExpressions: Seq[Expression])
   }
 }
 
+/**
+ * Returns a float between 0 and 1 representing the location of the closest point on a LineString to the given Point,
+ * as a fraction of 2d line length.
+ *
+ * @param inputExpressions
+ */
+case class ST_LineLocatePoint(inputExpressions: Seq[Expression])
+  extends InferredExpression(Functions.lineLocatePoint _) {
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
 case class ST_EndPoint(inputExpressions: Seq[Expression])
   extends InferredExpression(Functions.endPoint _) {
 
@@ -687,7 +702,7 @@ case class ST_SubDivideExplode(children: Seq[Expression])
 }
 
 case class ST_MakeLine(inputExpressions: Seq[Expression])
-  extends InferredExpression(InferrableFunction.allowRightNull(Functions.makeLine _)) {
+  extends InferredExpression(inferrableFunction2(Functions.makeLine), inferrableFunction1(Functions.makeLine)) {
 
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
@@ -1149,3 +1164,16 @@ case class ST_IsCollection(inputExpressions: Seq[Expression])
   }
 }
 
+/**
+ * Returns a text description of the validity of the geometry considering the specified flags.
+ * If flag not specified, it defaults to OGC SFS validity semantics.
+ *
+ * @param geom  The geometry to validate.
+ * @param flag The validation flags.
+ * @return A string describing the validity of the geometry.
+ */
+case class ST_IsValidReason(inputExpressions: Seq[Expression])
+  extends InferredExpression(inferrableFunction2(Functions.isValidReason), inferrableFunction1(Functions.isValidReason)) {
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = copy(inputExpressions = newChildren)
+}
