@@ -20,8 +20,9 @@ package org.apache.sedona.sql
 
 import org.apache.sedona.common.raster.MapAlgebra
 import org.apache.sedona.common.utils.RasterUtils
-import org.apache.spark.sql.SaveMode
-import org.apache.spark.sql.functions.{collect_list, expr}
+import org.apache.spark.sql.expressions.Window
+import org.apache.spark.sql.{Row, SaveMode}
+import org.apache.spark.sql.functions.{col, collect_list, expr, row_number}
 import org.geotools.coverage.grid.GridCoverage2D
 import org.junit.Assert.{assertEquals, assertNull, assertTrue}
 import org.locationtech.jts.geom.{Coordinate, Geometry}
@@ -400,6 +401,19 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
       assertTrue(expectedMetadata.equals(actualMetadata))
     }
 
+    it("Passed RS_SetValues with raster and implicit CRS transformation") {
+      val dfFile = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+      val df = dfFile.selectExpr("RS_FromGeoTiff(content) as raster", "ST_GeomFromWKT('POLYGON ((-8682522.873537656 4572703.890837922, -8673439.664183248 4572993.532747675, -8673155.57366801 4563873.2099182755, -8701890.325907696 4562931.7093397, -8682522.873537656 4572703.890837922))', 3857) as geom")
+      val resultDf = df.selectExpr("RS_SetValues(raster, 1, geom, 10, false) as result")
+
+      var actual = resultDf.selectExpr("RS_Value(result, ST_GeomFromWKT('POINT (-77.9146 37.8916)'))").first().get(0)
+      val expected = 10.0
+      assertEquals(expected, actual)
+
+      actual = resultDf.selectExpr("RS_Value(result, ST_GeomFromWKT('POINT (235749.0869 4200557.7397)', 26918))").first().get(0)
+      assertEquals(expected, actual)
+    }
+
     it("Passed RS_SetValues with empty raster") {
       var inputDf = Seq((Seq(1, 1, 1, 0, 0, 0, 1, 2, 3, 3, 5, 6, 7, 0, 0, 3, 0, 0, 3, 0, 0, 0, 0, 0, 0), Seq(11, 12, 13, 14, 15, 16, 17, 18, 19))).toDF("band","newValues")
       var df = inputDf.selectExpr("RS_AddBandFromArray(RS_MakeEmptyRaster(1, 5, 5, 0, 0, 1, -1, 0, 0, 0), band, 1, 0d) as raster", "newValues")
@@ -451,6 +465,9 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
       val actual = df.selectExpr("RS_BandNoDataValue(RS_SetBandNoDataValue(raster, 1, -999))").first().getDouble(0)
       val expected = -999
       assertEquals(expected, actual, 0.001d)
+
+      val actualNull = df.selectExpr("RS_BandNoDataValue(RS_SetBandNoDataValue(raster, 1, null))").first().get(0)
+      assertNull(actualNull)
     }
 
     it("Passed RS_SetBandNoDataValue with empty raster") {
@@ -490,10 +507,22 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
 
     it("Passed RS_Value with raster") {
       val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
-      val result = df.selectExpr("RS_Value(RS_FromGeoTiff(content), ST_Point(-13077301.685, 4002565.802))").first().getDouble(0)
+      val result = df.selectExpr("RS_Value(RS_FromGeoTiff(content), ST_SetSRID(ST_Point(-13077301.685, 4002565.802), 3857))").first().getDouble(0)
       assert(result == 255d)
     }
 
+    it("Passed RS_Value with raster and implicit CRS transformation") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+      val result = df.selectExpr("RS_Value(RS_FromGeoTiff(content), ST_GeomFromWKT('POINT (-77.9146 37.8916)', 4326))").first().get(0)
+      assert(result == 99d)
+    }
+
+    it("Passed RS_Value with raster and coordinates") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
+      val result = df.selectExpr("RS_Value(RS_FromGeoTiff(content), 4, 4, 1)").first().getDouble(0)
+      assert(result == 123d)
+    }
+
     it("Passed RS_Values should handle null values") {
       val result = sparkSession.sql("select RS_Values(null, null)").first().get(0)
       assert(result == null)
@@ -501,7 +530,7 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
 
     it("Passed RS_Values with raster") {
       val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
-      val result = df.selectExpr("RS_Values(RS_FromGeoTiff(content), array(ST_Point(-13077301.685, 4002565.802), null))").first().getList[Any](0)
+      val result = df.selectExpr("RS_Values(RS_FromGeoTiff(content), array(ST_SetSRID(ST_Point(-13077301.685, 4002565.802), 3857), null))").first().getList[Any](0)
       assert(result.size() == 2)
       assert(result.get(0) == 255d)
       assert(result.get(1) == null)
@@ -514,16 +543,94 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
       val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
       val points = sparkSession.createDataFrame(Seq(("POINT (-13077301.685 4002565.802)",1), ("POINT (0 0)",2)))
         .toDF("point", "id")
-        .withColumn("point", expr("ST_GeomFromText(point)"))
+        .withColumn("point", expr("ST_GeomFromText(point, 3857)"))
         .groupBy().agg(collect_list("point").alias("point"))
 
-      val result = df.crossJoin(points).selectExpr("RS_Values(RS_FromGeoTiff(content), point)").first().getList[Any](0)
+      val result = df.crossJoin(points).selectExpr("RS_Values(RS_FromGeoTiff(content), point, 1)").first().getList[Any](0)
 
       assert(result.size() == 2)
       assert(result.get(0) == 255d)
       assert(result.get(1) == null)
     }
 
+    it("Passed RS_Values with raster, serialized point array and implicit CRS transformation") {
+      // https://issues.apache.org/jira/browse/SEDONA-266
+      // A shuffle changes the internal type for the geometry array (point in this case) from GenericArrayData to UnsafeArrayData.
+      // UnsafeArrayData.array() throws UnsupportedOperationException.
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+      val points = sparkSession.createDataFrame(Seq(("POINT (-77.9146 37.8916)", 1), ("POINT (-100 100)", 2)))
+        .toDF("point", "id")
+        .withColumn("point", expr("ST_GeomFromText(point, 4326)"))
+        .groupBy().agg(collect_list("point").alias("point"))
+
+      val result = df.crossJoin(points).selectExpr("RS_Values(RS_FromGeoTiff(content), point, 1)").first().getList[Any](0)
+
+      assert(result.size() == 2)
+      assert(result.get(0) == 99d)
+      assert(result.get(1) == null)
+    }
+
+    it("Passed RS_Values with raster and Grid Coordinates") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
+      val result = df.selectExpr("RS_Values(RS_FromGeoTiff(content), array(1,2), array(3,2), 1)")
+        .first().getList[Double](0)
+
+      assert(result.size() == 2)
+      assert(result.get(0) == 132.0)
+      assert(result.get(1) == 132.0)
+    }
+
+    it("Passed RS_Clip with implicit geometry transformation") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+        .selectExpr("RS_FromGeoTiff(content) as raster",
+          "ST_GeomFromWKT('POLYGON ((-8682522.873537656 4572703.890837922, -8673439.664183248 4572993.532747675, -8673155.57366801 4563873.2099182755, -8701890.325907696 4562931.7093397, -8682522.873537656 4572703.890837922))', 3857) as geom")
+      val clippedDf = df.selectExpr("RS_Clip(raster, 1, geom, 200, false) as clipped")
+
+      val clippedMetadata = df.selectExpr("RS_Metadata(raster)").first().getSeq(0).slice(0, 9)
+      val originalMetadata = clippedDf.selectExpr("RS_Metadata(clipped)").first().getSeq(0).slice(0, 9)
+      assertTrue(originalMetadata.equals(clippedMetadata))
+
+      var actualValues = clippedDf.selectExpr(
+        "RS_Values(clipped, " +
+          "Array(ST_GeomFromWKT('POINT(223802 4.21769e+06)', 26918),ST_GeomFromWKT('POINT(224759 4.20453e+06)', 26918)," +
+          "ST_GeomFromWKT('POINT(237201 4.20429e+06)', 26918),ST_GeomFromWKT('POINT(237919 4.20357e+06)', 26918)," +
+          "ST_GeomFromWKT('POINT(254668 4.21769e+06)', 26918)), 1)"
+      ).first().get(0)
+      var expectedValues = Seq(null, null, 0.0, 0.0, null)
+      assertTrue(expectedValues.equals(actualValues))
+    }
+
+    it("Passed RS_Clip with raster") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+        .selectExpr("RS_FromGeoTiff(content) as raster",
+                    "ST_GeomFromWKT('POLYGON ((236722 4204770, 243900 4204770, 243900 4197590, 221170 4197590, 236722 4204770))', 26918) as geom")
+      val clippedDf = df.selectExpr("RS_Clip(raster, 1, geom, 200, false) as clipped")
+
+      val clippedMetadata = df.selectExpr("RS_Metadata(raster)").first().getSeq(0).slice(0, 9)
+      val originalMetadata = clippedDf.selectExpr("RS_Metadata(clipped)").first().getSeq(0).slice(0, 9)
+      assertTrue(originalMetadata.equals(clippedMetadata))
+
+      var actualValues = clippedDf.selectExpr(
+        "RS_Values(clipped, " +
+          "Array(ST_GeomFromWKT('POINT(223802 4.21769e+06)', 26918),ST_GeomFromWKT('POINT(224759 4.20453e+06)', 26918)," +
+          "ST_GeomFromWKT('POINT(237201 4.20429e+06)', 26918),ST_GeomFromWKT('POINT(237919 4.20357e+06)', 26918)," +
+          "ST_GeomFromWKT('POINT(254668 4.21769e+06)', 26918)), 1)"
+      ).first().get(0)
+      var expectedValues = Seq(null, null, 0.0, 0.0, null)
+      assertTrue(expectedValues.equals(actualValues))
+
+      val croppedDf = df.selectExpr("RS_Clip(raster, 1, geom, 200, false) as cropped")
+      actualValues = croppedDf.selectExpr(
+      "RS_Values(cropped, " +
+      "Array(ST_GeomFromWKT('POINT(236842 4.20465e+06)', 26918),ST_GeomFromWKT('POINT(236961 4.20453e+06)', 26918)," +
+      "ST_GeomFromWKT('POINT(237201 4.20429e+06)', 26918),ST_GeomFromWKT('POINT(237919 4.20357e+06)', 26918)," +
+      "ST_GeomFromWKT('POINT(223802 4.20465e+06)', 26918)), 1)"
+      ).first().get(0)
+      expectedValues = Seq(0.0, 0.0, 0.0, 0.0, null)
+      assertTrue(expectedValues.equals(actualValues))
+
+    }
+
     it("Passed RS_AsGeoTiff") {
       val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
       val resultRaw = df.selectExpr("RS_FromGeoTiff(content) as raster").first().get(0)
@@ -609,6 +716,20 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
       assertEquals(expected, result)
     }
 
+    it("Passed RS_Rotation") {
+      val df = sparkSession.sql("SELECT RS_MakeEmptyRaster(2, 10, 15, 1, 2, 1, -2, 1, 2, 0) as raster")
+      val actual = df.selectExpr("RS_Rotation(raster)").first().get(0)
+      val expected = -1.1071487177940904
+      assertEquals(expected, actual)
+    }
+
+    it("Passed RS_GeoTransform") {
+      val df = sparkSession.sql("SELECT RS_MakeEmptyRaster(2, 10, 15, 1, 2, 1, -2, 1, 2, 0) as raster")
+      val actual = df.selectExpr("RS_GeoTransform(raster)").first().getSeq(0)
+      val expected = mutable.ArraySeq(2.23606797749979, 2.23606797749979, -1.1071487177940904, -2.214297435588181, 1.0, 2.0)
+      assertTrue(expected.equals(actual))
+    }
+
     it("Passed RS_SkewX") {
       val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
       val result = df.selectExpr("RS_SkewX(RS_FromGeoTiff(content))").first().getDouble(0)
@@ -815,6 +936,82 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
       assertEquals(expected, actual)
     }
 
+    it("Passed RS_Union_Aggr") {
+      var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
+        .union(sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
+          .union(sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")))
+        .withColumn("raster", expr("RS_FromGeoTiff(content) as raster"))
+        .withColumn("index", row_number().over(Window.orderBy("raster")))
+        .selectExpr("raster", "index")
+
+      val dfTest = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
+        .selectExpr("RS_FromGeoTiff(content) as raster")
+
+      df = df.selectExpr("RS_Union_aggr(raster, index) as rasters")
+
+      val actualBands = df.selectExpr("RS_NumBands(rasters)").first().get(0)
+      val expectedBands = 3
+      assertEquals(expectedBands, actualBands)
+
+      val actualMetadata = df.selectExpr("RS_Metadata(rasters)").first().getSeq(0).slice(0, 9)
+      val expectedMetadata = dfTest.selectExpr("RS_Metadata(raster)").first().getSeq(0).slice(0, 9)
+      assertTrue(expectedMetadata.equals(actualMetadata))
+    }
+
+    it("Passed RS_ZonalStats") {
+      var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+      df = df.selectExpr("RS_FromGeoTiff(content) as raster", "ST_GeomFromWKT('POLYGON ((236722 4204770, 243900 4204770, 243900 4197590, 221170 4197590, 236722 4204770))', 26918) as geom")
+      var actual = df.selectExpr("RS_ZonalStats(raster, geom, 1, 'sum', true)").first().get(0)
+      assertEquals(1.0690406E7, actual)
+
+      actual = df.selectExpr("RS_ZonalStats(raster, geom, 1, 'count', false)").first().get(0)
+      assertEquals(184792.0, actual)
+
+      actual = df.selectExpr("RS_ZonalStats(raster, geom, 1, 'mean', false)").first().get(0)
+      assertEquals(57.851021689230684, actual)
+
+      actual = df.selectExpr("RS_ZonalStats(raster, geom, 1, 'variance')").first().get(0)
+      assertEquals(8488.448098819916, actual)
+
+      actual = df.selectExpr("RS_ZonalStats(raster, geom, 'sd')").first().get(0)
+      assertEquals(92.13277429243035, actual)
+    }
+
+    it("Passed RS_ZonalStats - Raster with no data") {
+      var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/raster_with_no_data/test5.tiff")
+      df = df.selectExpr("RS_FromGeoTiff(content) as raster", "ST_GeomFromWKT('POLYGON((-167.750000 87.750000, -155.250000 87.750000, -155.250000 40.250000, -180.250000 40.250000, -167.750000 87.750000))', 4326) as geom")
+      var actual = df.selectExpr("RS_ZonalStats(raster, geom, 1, 'sum', true)").first().get(0)
+      assertEquals(3213526.0, actual)
+
+      actual = df.selectExpr("RS_ZonalStats(raster, geom, 1, 'count', true)").first().get(0)
+      assertEquals(14184.0, actual)
+
+      actual = df.selectExpr("RS_ZonalStats(raster, geom, 1, 'mean', false)").first().get(0)
+      assertEquals(226.49605300253515, actual)
+
+      actual = df.selectExpr("RS_ZonalStats(raster, geom, 1, 'variance')").first().get(0)
+      assertEquals(5606.423398599913, actual)
+
+      actual = df.selectExpr("RS_ZonalStats(raster, geom, 'sd')").first().get(0)
+      assertEquals(74.87605357255357, actual)
+    }
+
+    it("Passed RS_ZonalStatsAll") {
+      var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+      df = df.selectExpr("RS_FromGeoTiff(content) as raster", "ST_GeomFromWKT('POLYGON ((-8673439.6642 4572993.5327, -8673155.5737 4563873.2099, -8701890.3259 4562931.7093, -8682522.8735 4572703.8908, -8673439.6642 4572993.5327))', 3857) as geom")
+      val actual = df.selectExpr("RS_ZonalStatsAll(raster, geom, 1, true)").first().get(0)
+      val expected = Seq(184792.0, 1.0690406E7, 57.851021689230684, 0.0, 0.0, 92.13277429243035, 8488.448098819916, 0.0, 255.0)
+      assertTrue(expected.equals(actual))
+    }
+
+    it("Passed RS_ZonalStatsAll - Raster with no data") {
+      var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/raster_with_no_data/test5.tiff")
+      df = df.selectExpr("RS_FromGeoTiff(content) as raster", "ST_GeomFromWKT('POLYGON((-167.750000 87.750000, -155.250000 87.750000, -155.250000 40.250000, -180.250000 40.250000, -167.750000 87.750000))', 0) as geom")
+      val actual = df.selectExpr("RS_ZonalStatsAll(raster, geom, 1, true)").first().get(0)
+      val expected = Seq(14184.0, 3213526.0, 226.55992667794473, 255.0, 255.0, 74.87605357255357, 5606.423398599913, 1.0, 255.0)
+      assertTrue(expected.equals(actual))
+    }
+
     it("Passed RS_SummaryStats with raster") {
       var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/raster_with_no_data/test5.tiff")
       df = df.selectExpr("RS_FromGeoTiff(content) as raster")
@@ -867,6 +1064,34 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
       assertEquals(expectedY, actualCoordinates.y, 1e-5)
     }
 
+    it("Passed RS_PixelAsPoints with empty raster") {
+      val widthInPixel = 5
+      val heightInPixel = 10
+      val upperLeftX = 123.19
+      val upperLeftY = -12
+      val cellSize = 4
+      val numBands = 2
+      val result = sparkSession.sql(s"SELECT RS_PixelAsPoints(RS_MakeEmptyRaster($numBands, $widthInPixel, $heightInPixel, $upperLeftX, $upperLeftY, $cellSize), 1)").first().getList(0);
+      val expected = "[POINT (127.19000244140625 -12),0.0,2,1]"
+      assertEquals(expected, result.get(1).toString)
+    }
+
+    it("Passed RS_PixelAsPoints with raster") {
+      var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
+      df = df.selectExpr("RS_FromGeoTiff(content) as raster")
+
+      var result = df.selectExpr(
+        "explode(RS_PixelAsPoints(raster, 1)) as exploded"
+      ).selectExpr(
+        "exploded.geom as geom",
+        "exploded.value as value",
+        "exploded.x as x",
+        "exploded.y as y"
+      )
+
+      assert(result.count() == 264704)
+    }
+
     it("Passed RS_PixelAsPolygon with raster") {
       val widthInPixel = 5
       val heightInPixel = 10
@@ -879,6 +1104,35 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
       assertEquals(expected, result)
     }
 
+    it("Passed RS_PixelAsPolygons with empty raster") {
+      val widthInPixel = 5
+      val heightInPixel = 10
+      val upperLeftX = 123.19
+      val upperLeftY = -12
+      val cellSize = 4
+      val numBands = 2
+      val result = sparkSession.sql(s"SELECT RS_PixelAsPolygons(RS_MakeEmptyRaster($numBands, $widthInPixel, $heightInPixel, $upperLeftX, $upperLeftY, $cellSize), 1)").first().getList(0);
+      val expected = "[POLYGON ((127.19000244140625 -20, 131.19000244140625 -20, 131.19000244140625 -24, 127.19000244140625 -24, 127.19000244140625 -20)),0.0,2,3]"
+      assertEquals(expected, result.get(11).toString)
+    }
+
+
+    it("Passed RS_PixelAsPolygons with raster") {
+      var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
+      df = df.selectExpr("RS_FromGeoTiff(content) as raster")
+
+      var result = df.selectExpr(
+        "explode(RS_PixelAsPolygons(raster, 1)) as exploded"
+      ).selectExpr(
+        "exploded.geom as geom",
+        "exploded.value as value",
+        "exploded.x as x",
+        "exploded.y as y"
+      )
+
+      assert(result.count() == 264704)
+    }
+
     it("Passed RS_PixelAsCentroid with raster") {
       val widthInPixel = 12
       val heightInPixel = 13
@@ -891,6 +1145,34 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
       assertEquals(expected, result)
     }
 
+    it("Passed RS_PixelAsCentroids with empty raster") {
+      val widthInPixel = 12
+      val heightInPixel = 13
+      val upperLeftX = 240
+      val upperLeftY = -193
+      val cellSize = 9
+      val numBands = 2
+      val result = sparkSession.sql(s"SELECT RS_PixelAsCentroids(RS_MakeEmptyRaster($numBands, $widthInPixel, $heightInPixel, $upperLeftX, $upperLeftY, $cellSize), 1)").first().getList(0);
+      val expected = "[POINT (253.5 -215.5),0.0,2,3]"
+      assertEquals(expected, result.get(25).toString)
+    }
+
+    it("Passed RS_PixelAsCentroids with raster") {
+      var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
+      df = df.selectExpr("RS_FromGeoTiff(content) as raster")
+
+      var result = df.selectExpr(
+        "explode(RS_PixelAsCentroids(raster, 1)) as exploded"
+      ).selectExpr(
+        "exploded.geom as geom",
+        "exploded.value as value",
+        "exploded.x as x",
+        "exploded.y as y"
+      )
+
+      assert(result.count() == 264704)
+    }
+
     it("Passed RS_RasterToWorldCoordX with raster") {
       var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
       df = df.selectExpr("RS_FromGeoTiff(content) as raster")
@@ -905,6 +1187,14 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
       assertEquals(4021262.7487925636, result, 0.5d)
     }
 
+    it("Passed RS_RasterToWorldCoord with raster") {
+      var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
+      df = df.selectExpr("RS_FromGeoTiff(content) as raster")
+      val result = df.selectExpr("RS_RasterToWorldCoord(raster, 1, 1)").first().get(0).toString
+      val expected = "POINT (-13095818 4021262.75)"
+      assertEquals(expected, result)
+    }
+
     it("Passed RS_WorldToRasterCoord with raster") {
       var df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/test1.tiff")
       df = df.selectExpr("RS_FromGeoTiff(content) as raster")
@@ -1006,6 +1296,174 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
       assertEquals("UNSIGNED_8BITS", result)
     }
 
+    it("Passed RS_Tile - in-db raster") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/raster_with_no_data/test5.tiff")
+        .selectExpr("RS_FromGeoTiff(content) as raster")
+      val resultDf = df.selectExpr("RS_Tile(raster, 100, 100) as tiles")
+      val result = resultDf.first().get(0)
+      assert(result.isInstanceOf[mutable.WrappedArray[GridCoverage2D]])
+      val tiles = result.asInstanceOf[mutable.WrappedArray[GridCoverage2D]]
+      assert(tiles.exists(tile => tile.getRenderedImage.getWidth < 100 || tile.getRenderedImage.getHeight < 100))
+      tiles.foreach { tile =>
+        assert(tile.getRenderedImage.getData != null)
+        assert(RasterUtils.getNoDataValue(tile.getSampleDimension(0)) == 0)
+        tile.dispose(true)
+      }
+      val rsValuesDf = resultDf.selectExpr("explode(tiles) as tile")
+        .selectExpr("RS_Value(tile, ST_Centroid(RS_Envelope(tile))) as value")
+        .withColumn("is_non_null", expr("value is not null"))
+      assert(rsValuesDf.count() == 120)
+      assert((90 to 110) contains rsValuesDf.where("is_non_null").count())
+    }
+
+    it("Passed RS_Tile - in-db raster with padding") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/raster_with_no_data/test5.tiff")
+        .selectExpr("RS_FromGeoTiff(content) as raster")
+      val resultDf = df.selectExpr("RS_Tile(raster, null, 100, 100, true, 3) as tiles")
+      val result = resultDf.first().get(0)
+      assert(result.isInstanceOf[mutable.WrappedArray[GridCoverage2D]])
+      val tiles = result.asInstanceOf[mutable.WrappedArray[GridCoverage2D]]
+      assert(tiles.exists(tile => RasterUtils.getNoDataValue(tile.getSampleDimension(0)) == 3))
+      tiles.foreach { tile =>
+        assert(tile.getRenderedImage.getData != null)
+        assert(tile.getRenderedImage.getWidth == 100 && tile.getRenderedImage.getHeight == 100)
+        tile.dispose(true)
+      }
+    }
+
+    it("Passed RS_Tile - in-db raster with band index") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+        .selectExpr("RS_FromGeoTiff(content) as raster")
+      val resultDf = df.selectExpr("RS_Tile(raster, array(2), 100, 100, false) as tiles")
+      val result = resultDf.first().get(0)
+      assert(result.isInstanceOf[mutable.WrappedArray[GridCoverage2D]])
+      val tiles = result.asInstanceOf[mutable.WrappedArray[GridCoverage2D]]
+      tiles.foreach { tile =>
+        assert(tile.getRenderedImage.getData != null)
+        assert(tile.getNumSampleDimensions == 1)
+        tile.dispose(true)
+      }
+    }
+
+    it("Passed RS_Tile - in-db raster with band indices") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+        .selectExpr("RS_FromGeoTiff(content) as raster")
+      val resultDf = df.selectExpr("RS_Tile(raster, array(2, 1), 100, 100) as tiles")
+      val result = resultDf.first().get(0)
+      assert(result.isInstanceOf[mutable.WrappedArray[GridCoverage2D]])
+      val tiles = result.asInstanceOf[mutable.WrappedArray[GridCoverage2D]]
+      tiles.foreach { tile =>
+        assert(tile.getRenderedImage.getData != null)
+        assert(tile.getNumSampleDimensions == 2)
+        tile.dispose(true)
+      }
+    }
+
+    it("Passed RS_TileExplode - in-db raster") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/raster_with_no_data/test5.tiff")
+        .selectExpr("RS_FromGeoTiff(content) as raster")
+      val resultDf = df.selectExpr("RS_TileExplode(raster, 100, 100) AS (x, y, tile)")
+      val result = resultDf.collect()
+      assert(result.length == 120)
+      result.foreach { row =>
+        val tile = row.getAs[GridCoverage2D]("tile")
+        assert(tile.isInstanceOf[GridCoverage2D])
+        assert(tile.getRenderedImage.getData != null)
+        tile.dispose(true)
+      }
+    }
+
+    it("Passed RS_TileExplode - in-db raster with padding") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/raster_with_no_data/test5.tiff")
+        .selectExpr("RS_FromGeoTiff(content) as raster")
+      Seq("3", "3.0", "3.0d").foreach { noData =>
+        val resultDf = df.selectExpr(s"RS_TileExplode(raster, 100, 100, true, $noData) AS (x, y, tile)")
+        val result = resultDf.collect()
+        val tiles = result.map(_.getAs[GridCoverage2D]("tile"))
+        assert(tiles.exists(tile => RasterUtils.getNoDataValue(tile.getSampleDimension(0)) == 3))
+        tiles.foreach { tile =>
+          assert(tile.getRenderedImage.getData != null)
+          assert(tile.getRenderedImage.getWidth == 100 && tile.getRenderedImage.getHeight == 100)
+          tile.dispose(true)
+        }
+      }
+    }
+
+    it("Passed RS_TileExplode - in-db raster with band index") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+        .selectExpr("RS_FromGeoTiff(content) as raster")
+      val resultDf = df.selectExpr("RS_TileExplode(raster, 2, 100, 100, true, 5) AS (x, y, tile)")
+      val result = resultDf.collect()
+      val tiles = result.map(_.getAs[GridCoverage2D]("tile"))
+      assert(tiles.exists(tile => RasterUtils.getNoDataValue(tile.getSampleDimension(0)) == 5))
+      tiles.foreach { tile =>
+        assert(tile.getRenderedImage.getData != null)
+        assert(tile.getNumSampleDimensions == 1)
+        tile.dispose(true)
+      }
+    }
+
+    it("Passed RS_TileExplode - in-db raster with band indices") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif")
+        .selectExpr("RS_FromGeoTiff(content) as raster")
+      val resultDf = df.selectExpr("RS_TileExplode(raster, array(2, 1), 100, 100) AS (x, y, tile)")
+      val result = resultDf.collect()
+      val tiles = result.map(_.getAs[GridCoverage2D]("tile"))
+      tiles.foreach { tile =>
+        assert(tile.getRenderedImage.getData != null)
+        assert(tile.getNumSampleDimensions == 2)
+        tile.dispose(true)
+      }
+    }
+
+    it("Passed RS_MapAlgebra with two raster columns") {
+      var df = sparkSession.read.format("binaryFile")
+        .option("recursiveFileLookup", "true")
+        .option("pathGlobFilter", "*.tif*")
+        .load(resourceFolder + "raster")
+        .selectExpr("path", "RS_FromGeoTiff(content) as rast")
+      df = df.as("r1")
+        .join(df.as("r2"), col("r1.path") === col("r2.path"), "inner")
+        .select(col("r1.rast").alias("rast0"), col("r2.rast").alias("rast1"), col("r1.path"))
+      Seq(null, "b", "s", "i", "f", "d").foreach { pixelType =>
+        val pixelTypeExpr = if (pixelType == null) null else s"'$pixelType'"
+        val dfResult = df.withColumn("rast_2", expr(s"RS_MapAlgebra(rast0, rast1, $pixelTypeExpr, 'out[0] = rast0[0] * 0.5 + rast1[0] * 0.5;', null)"))
+          .select("path", "rast0", "rast1", "rast_2")
+        dfResult.collect().foreach { row =>
+          val rast0 = row.getAs[GridCoverage2D]("rast0")
+          val rast1 = row.getAs[GridCoverage2D]("rast1")
+          val resultRaster = row.getAs[GridCoverage2D]("rast_2")
+          assert(rast0.getGridGeometry.getGridToCRS2D == resultRaster.getGridGeometry.getGridToCRS2D)
+          val dataType = pixelType match {
+            case null => rast0.getRenderedImage.getSampleModel.getDataType
+            case _ => RasterUtils.getDataTypeCode(pixelType)
+          }
+          assert(resultRaster.getRenderedImage.getSampleModel.getDataType == dataType)
+          val noDataValue = RasterUtils.getNoDataValue(resultRaster.getSampleDimension(0))
+          assert(noDataValue.isNaN)
+          val band0 = MapAlgebra.bandAsArray(rast0, 1)
+          val band1 = MapAlgebra.bandAsArray(rast1, 1)
+          val bandResult = MapAlgebra.bandAsArray(resultRaster, 1)
+          assert(band0.size == bandResult.size)
+          for (i <- band0.indices) {
+            pixelType match {
+              case "b" => assert(((band0(i) * 0.5 + band1(i) * 0.5).toInt & 0xFF) == bandResult(i).toInt)
+              case "s" => assert((band0(i) * 0.5 + band1(i) * 0.5).toShort == bandResult(i))
+              case "i" | null => assert((band0(i) * 0.5 + band1(i) * 0.5).toInt == bandResult(i))
+              case "f" | "d" =>
+                if (band0(i) != 0) {
+                  assert((band0(i) * 0.5 + band1(i) * 0.5) == bandResult(i))
+                } else {
+                  // If the source image has NoDataContainer.GC_NODATA property, Jiffle may convert nodata values in
+                  // source raster to NaN.
+                  assert(bandResult(i) == 0 || bandResult(i).isNaN)
+                }
+            }
+          }
+        }
+      }
+    }
+
     it("Passed RS_MapAlgebra") {
       val df = sparkSession.read.format("binaryFile")
         .option("recursiveFileLookup", "true")
@@ -1162,5 +1620,44 @@ class rasteralgebraTest extends TestBaseScala with BeforeAndAfter with GivenWhen
         assertEquals(expectedMetadata(i), rasterMetadata(i), 1e-6)
       }
     }
+
+    it("Passed RS_FromNetCDF with NetCDF classic") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/netcdf/test.nc")
+      val rasterDf = df.selectExpr("RS_FromNetCDF(content, 'O3') as raster")
+      val expectedMetadata = Seq(4.9375, 50.9375, 80, 48, 0.125, -0.125, 0, 0, 0, 4)
+      val actualMetadata = rasterDf.selectExpr("RS_Metadata(raster) as metadata").first().getSeq[Double](0)
+
+      for (i <- expectedMetadata.indices) {
+        assertEquals(expectedMetadata(i), actualMetadata(i), 1e-6)
+      }
+
+      val expectedFirstVal = 60.95357131958008
+      val actualFirstVal = rasterDf.selectExpr("RS_Value(raster, 0, 0, 1) as raster").first().getDouble(0)
+      assertEquals(expectedFirstVal, actualFirstVal, 1e-6)
+    }
+
+    it("Passed RS_FromNetCDF with NetCDF classic long form") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/netcdf/test.nc")
+      val rasterDf = df.selectExpr("RS_FromNetCDF(content, 'O3', 'lon', 'lat') as raster")
+      val expectedMetadata = Seq(4.9375, 50.9375, 80, 48, 0.125, -0.125, 0, 0, 0, 4)
+      val actualMetadata = rasterDf.selectExpr("RS_Metadata(raster) as metadata").first().getSeq[Double](0)
+
+      for (i <- expectedMetadata.indices) {
+        assertEquals(expectedMetadata(i), actualMetadata(i), 1e-6)
+      }
+
+      val expectedFirstVal = 60.95357131958008
+      val actualFirstVal = rasterDf.selectExpr("RS_Value(raster, 0, 0, 1) as raster").first().getDouble(0)
+      assertEquals(expectedFirstVal, actualFirstVal, 1e-6)
+    }
+
+    it("Passed RS_NetCDFInfo") {
+      val df = sparkSession.read.format("binaryFile").load(resourceFolder + "raster/netcdf/test.nc")
+      val recordInfo = df.selectExpr("RS_NetCDFInfo(content) as record_info").first().getString(0)
+      val expectedRecordInfo = "O3(time=2, z=2, lat=48, lon=80)\n" +
+        "\n" +
+        "NO2(time=2, z=2, lat=48, lon=80)"
+      assertEquals(expectedRecordInfo, recordInfo)
+    }
   }
 }
